<% if object.has_tag?(:example) %>
  <div class="examples">
    <h4 class="tag_title">Examples:</h4>
    <% object.tags(:example).each do |tag| %>
      <% unless tag.name.empty? %>
        <h5 class="example_title"><%= htmlify_line(tag.name) %></h5>
      <% end %>
      <pre class="example code"><code><%= html_syntax_highlight(tag.text) %></code></pre>
    <% end %>
  </div>
<% end %>