body {
	margin: 0;
	font-family: "Lucida Sans", "Lucida Grande", Verdana, Arial, sans-serif;
	font-size: 13px;
	height: 101%;
	overflow-x: hidden;
	background: #fafafa;
}

h1 {
	padding: 12px 10px;
	padding-bottom: 0;
	margin: 0;
	font-size: 1.4em;
}
.clear {
	clear: both;
}
.fixed_header {
	position: fixed;
	background: #fff;
	width: 100%;
	padding-bottom: 10px;
	margin-top: 0;
	top: 0;
	z-index: 9999;
	height: 70px;
}
#search {
	position: absolute;
	right: 5px;
	top: 9px;
	padding-left: 24px;
}
#noresults {
	padding: 7px 12px;
	background: #fff;
}
#content.insearch #search,
#content.insearch #noresults {
	background: url(data:image/gif;base64,R0lGODlhEAAQAPYAAP///wAAAPr6+pKSkoiIiO7u7sjIyNjY2J6engAAAI6OjsbGxjIyMlJSUuzs7KamppSUlPLy8oKCghwcHLKysqSkpJqamvT09Pj4+KioqM7OzkRERAwMDGBgYN7e3ujo6Ly8vCoqKjY2NkZGRtTU1MTExDw8PE5OTj4+PkhISNDQ0MrKylpaWrS0tOrq6nBwcKysrLi4uLq6ul5eXlxcXGJiYoaGhuDg4H5+fvz8/KKiohgYGCwsLFZWVgQEBFBQUMzMzDg4OFhYWBoaGvDw8NbW1pycnOLi4ubm5kBAQKqqqiQkJCAgIK6urnJyckpKSjQ0NGpqatLS0sDAwCYmJnx8fEJCQlRUVAoKCggICLCwsOTk5ExMTPb29ra2tmZmZmhoaNzc3KCgoBISEiIiIgAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH/C05FVFNDQVBFMi4wAwEAAAAh/hpDcmVhdGVkIHdpdGggYWpheGxvYWQuaW5mbwAh+QQJCAAAACwAAAAAEAAQAAAHaIAAgoMgIiYlg4kACxIaACEJCSiKggYMCRselwkpghGJBJEcFgsjJyoAGBmfggcNEx0flBiKDhQFlIoCCA+5lAORFb4AJIihCRbDxQAFChAXw9HSqb60iREZ1omqrIPdJCTe0SWI09GBACH5BAkIAAAALAAAAAAQABAAAAdrgACCgwc0NTeDiYozCQkvOTo9GTmDKy8aFy+NOBA7CTswgywJDTIuEjYFIY0JNYMtKTEFiRU8Pjwygy4ws4owPyCKwsMAJSTEgiQlgsbIAMrO0dKDGMTViREZ14kYGRGK38nHguHEJcvTyIEAIfkECQgAAAAsAAAAABAAEAAAB2iAAIKDAggPg4iJAAMJCRUAJRIqiRGCBI0WQEEJJkWDERkYAAUKEBc4Po1GiKKJHkJDNEeKig4URLS0ICImJZAkuQAhjSi/wQyNKcGDCyMnk8u5rYrTgqDVghgZlYjcACTA1sslvtHRgQAh+QQJCAAAACwAAAAAEAAQAAAHZ4AAgoOEhYaCJSWHgxGDJCQARAtOUoQRGRiFD0kJUYWZhUhKT1OLhR8wBaaFBzQ1NwAlkIszCQkvsbOHL7Y4q4IuEjaqq0ZQD5+GEEsJTDCMmIUhtgk1lo6QFUwJVDKLiYJNUd6/hoEAIfkECQgAAAAsAAAAABAAEAAAB2iAAIKDhIWGgiUlh4MRgyQkjIURGRiGGBmNhJWHm4uen4ICCA+IkIsDCQkVACWmhwSpFqAABQoQF6ALTkWFnYMrVlhWvIKTlSAiJiVVPqlGhJkhqShHV1lCW4cMqSkAR1ofiwsjJyqGgQAh+QQJCAAAACwAAAAAEAAQAAAHZ4AAgoOEhYaCJSWHgxGDJCSMhREZGIYYGY2ElYebi56fhyWQniSKAKKfpaCLFlAPhl0gXYNGEwkhGYREUywag1wJwSkHNDU3D0kJYIMZQwk8MjPBLx9eXwuETVEyAC/BOKsuEjYFhoEAIfkECQgAAAAsAAAAABAAEAAAB2eAAIKDhIWGgiUlh4MRgyQkjIURGRiGGBmNhJWHm4ueICImip6CIQkJKJ4kigynKaqKCyMnKqSEK05StgAGQRxPYZaENqccFgIID4KXmQBhXFkzDgOnFYLNgltaSAAEpxa7BQoQF4aBACH5BAkIAAAALAAAAAAQABAAAAdogACCg4SFggJiPUqCJSWGgkZjCUwZACQkgxGEXAmdT4UYGZqCGWQ+IjKGGIUwPzGPhAc0NTewhDOdL7Ykji+dOLuOLhI2BbaFETICx4MlQitdqoUsCQ2vhKGjglNfU0SWmILaj43M5oEAOwAAAAAAAAAAAA==)
		no-repeat center left;
}
#full_list {
	padding: 0;
	list-style: none;
	margin-left: 0;
	margin-top: 80px;
	font-size: 1.1em;
}
#full_list ul {
	padding: 0;
}
#full_list li {
	padding: 0;
	margin: 0;
	list-style: none;
}
#full_list li .item {
	padding: 5px 5px 5px 12px;
}
#content.insearch #noresults {
	margin-left: 7px;
}
#full_list li {
	color: #666;
	cursor: normal;
	white-space: nowrap;
}
#full_list li.collapsed ul {
	display: none;
}
#full_list li a.toggle {
	cursor: default;
	position: relative;
	left: -5px;
	top: 4px;
	text-indent: -999px;
	width: 10px;
	height: 9px;
	margin-left: -10px;
	display: block;
	float: left;
	background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAASCAYAAABb0P4QAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAAK8AAACvABQqw0mAAAABx0RVh0U29mdHdhcmUAQWRvYmUgRmlyZXdvcmtzIENTM5jWRgMAAAAVdEVYdENyZWF0aW9uIFRpbWUAMy8xNC8wOeNZPpQAAAE2SURBVDiNrZTBccIwEEXfelIAHUA6CZ24BGaWO+FuzZAK4k6gg5QAdGAq+Bxs2Yqx7BzyL7Llp/VfzZeQhCTc/ezuGzKKnKSzpCxXJM8fwNXda3df5RZETlIt6YUzSQDs93sl8w3wBZxCCE10GM1OcWbWjB2mWgEH4Mfdyxm3PSepBHibgQE2wLe7r4HjEidpnXMYdQPKEMJcsZ4zs2POYQOcaPfwMVOo58zsAdMt18BuoVDPxUJRacELbXv3hUIX2vYmOUvi8C8ydz/ThjXrqKqqLbDIAdsCKBd+Wo7GWa7o9qzOQHVVVXeAbs+yHHCH4aTsaCOQqunmUy1yBUAXkdMIfMlgF5EXLo2OpV/c/Up7jG4hhHcYLgWzAZXUc2b2ixsfvc/RmNNfOXD3Q/oeL9axJE1yT9IOoUu6MGUkAAAAAElFTkSuQmCC)
		no-repeat bottom left;
}
#full_list li.collapsed a.toggle {
	cursor: default;
	background-position: top left;
}
#full_list li.deprecated {
	text-decoration: line-through;
	font-style: italic;
}
#full_list li.odd {
	background: #f0f0f0;
}
#full_list li.even {
	background: #fafafa;
}
#full_list .item:hover {
	background: #ddd;
}
#full_list li small:before {
	content: "(";
}
#full_list li small:after {
	content: ")";
}
#full_list li small.search_info {
	display: none;
}
a,
a:visited {
	text-decoration: none;
	color: #05a;
}
#full_list li.clicked > .item {
	background: #05a;
	color: #ccc;
}
#full_list li.clicked > .item a,
#full_list li.clicked > .item a:visited {
	color: #eee;
}
#full_list li.clicked > .item a.toggle {
	opacity: 0.5;
	background-position: bottom right;
}
#full_list li.collapsed.clicked a.toggle {
	background-position: top right;
}
#search input {
	border: 1px solid #bbb;
	border-radius: 3px;
}
#full_list_nav {
	margin-left: 10px;
	font-size: 0.9em;
	display: block;
	color: #aaa;
}
#full_list_nav a,
#nav a:visited {
	color: #358;
}
#full_list_nav a:hover {
	background: transparent;
	color: #5af;
}
#full_list_nav span:after {
	content: " | ";
}
#full_list_nav span:last-child:after {
	content: "";
}

#content h1 {
	margin-top: 0;
}
#full_list li small {
	display: block;
	font-size: 0.8em;
}
#full_list li small:before {
	content: "";
}
#full_list li small:after {
	content: "";
}
#full_list li small.search_info {
	display: none;
}
#search {
	width: 170px;
	position: static;
	margin: 3px;
	margin-left: 10px;
	font-size: 0.9em;
	color: #666;
	padding-left: 0;
	padding-right: 24px;
}
#content.insearch #search {
	background-position: center right;
}
#search input {
	width: 110px;
}

#full_list.insearch ul {
	display: block;
}
#full_list.insearch .item {
	display: none;
}
#full_list.insearch .found {
	display: block;
	padding-left: 11px;
}
#full_list.insearch li a.toggle {
	display: none;
}
#full_list.insearch li small.search_info {
	display: block;
}
