<h1>Class: A
  <span class="abstract note title">Abstract</span>
  
  
</h1>
<div class="box_info">
  
  <dl>
    <dt>Inherits:</dt>
    <dd>
      <span class="inheritName">B</span>
      
        <ul class="fullTree">
          <li>Object</li>
          
            <li class="next">B</li>
          
            <li class="next">A</li>
          
        </ul>
        <a href="#" class="inheritanceTree">show all</a>
      
    </dd>
  </dl>
  

  
  
  
  
  

  

  
  <dl>
    <dt>Defined in:</dt>
    <dd>(stdin)</dd>
  </dl>
  
</div>

<h2>Overview</h2><div class="docstring">
  <div class="discussion">
    <div class="note abstract">
  <strong>This class is abstract.</strong>
  <div class='inline'>override this class</div>
</div>
Comments

  </div>
</div>
<div class="tags">
  

  <p class="tag_title">See Also:</p>
  <ul class="see">
    
      <li>A</li>
    
      <li><a href="https://example.com" target="_parent" title="Example">Example</a></li>
    
  </ul>
<p class="tag_title">Author:</p>
<ul class="author">
  
    <li>
      
      
      
      
        
        <div class='inline'>Test</div>
      
    </li>
  
</ul>
<p class="tag_title">Version:</p>
<ul class="version">
  
    <li>
      
      
      
      
        
        <div class='inline'>1.0</div>
      
    </li>
  
</ul>

</div><div id="subclasses">
  <h2>Direct Known Subclasses</h2>
  <p class="children">C</p>
</div>







  
    <h2>
      Instance Method Summary
      <small><a href="#" class="summary_toggle">collapse</a></small>
    </h2>

    <ul class="summary">
      
        <li class="public deprecated">
  <span class="summary_signature">
    
      <a  title="#a (instance method)">#<strong>a</strong>  </a>
    

    
  </span>
  
  
  
  
  
  <span class="deprecated note title">deprecated</span>
  

  
    <span class="summary_desc"><strong>Deprecated.</strong> <div class='inline'></div></span>
  
</li>

      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#initialize (instance method)">#<strong>initialize</strong>(test)  &#x21d2; A </a>
    

    
  </span>
  
  
    <span class="note title constructor">constructor</span>
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>constructor method!.</div></span>
  
</li>

      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#method_missing (instance method)">#<strong>method_missing</strong>(*args)  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>HI.</div></span>
  
</li>

      
    </ul>
  


  <div id="constructor_details" class="method_details_list">
  <h2>Constructor Details</h2>
  
    <div class="method_details first">
  <h3 class="signature first" id="initialize-instance_method">
  
    #<strong>initialize</strong>(test)  &#x21d2; <tt>A</tt> 
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    constructor method!

  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


15</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 15</span>

def initialize(test) end</pre>
    </td>
  </tr>
</table>
</div>
  
</div>
<div id="method_missing_details" class="method_details_list">
  <h2>Dynamic Method Handling</h2>
  <p class="notice this">
    This class handles dynamic methods through the <tt>method_missing</tt> method
    
  </p>
  
    <div class="method_details first">
  <h3 class="signature first" id="method_missing-instance_method">
  
    #<strong>method_missing</strong>(*args)  
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    HI

  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


10</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 10</span>

def method_missing(*args) end</pre>
    </td>
  </tr>
</table>
</div>
  
</div>


  <div id="instance_method_details" class="method_details_list">
    <h2>Instance Method Details</h2>

    
      <div class="method_details first">
  <h3 class="signature first" id="a-instance_method">
  
    #<strong>a</strong>  
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    <div class="note deprecated"><strong>Deprecated.</strong> <div class='inline'></div></div>


  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


12</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 12</span>

def a; end</pre>
    </td>
  </tr>
</table>
</div>
    
  </div>
