<h1>Method: #m</h1>
<div class="box_info">
  <dl>
    <dt class="">Defined in:</dt>
    <dd class="">
    (stdin)
    </dd>
  </dl>
</div>

<div class="method_details_list">
  <div id="method_details">
    <div class="method_details first">
  <h3 class="signature first" id="m-instance_method">
  
    
      <span class="overload">#<strong>m</strong>(x, y)   <span class="extras">(private)</span></span>
    
      <span class="overload">#<strong>m</strong>(x, y, z)   <span class="extras">(private)</span></span>
    
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    Method comments

  </div>
</div>
<div class="tags">
  
  <p class="tag_title">Overloads:</p>
  <ul class="overload">
    
      
      <li class="overload_item">
        <span class="signature">#<strong>m</strong>(x, y)  </span>
        <div class="docstring">
  <div class="discussion">
    Overload docstring

  </div>
</div>
<div class="tags">
  <p class="tag_title">Parameters:</p>
<ul class="param">
  
    <li>
      
        <span class='name'>x</span>
      
      
        <span class='type'>(<tt>String</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>parameter x</div>
      
    </li>
  
    <li>
      
        <span class='name'>y</span>
      
      
        <span class='type'>(<tt>Boolean</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>parameter y</div>
      
    </li>
  
</ul>


</div>
      </li>
    
      
      <li class="overload_item">
        <span class="signature">#<strong>m</strong>(x, y, z)  </span>
        <div class="docstring">
  <div class="discussion">
    

  </div>
</div>
<div class="tags">
  <p class="tag_title">Parameters:</p>
<ul class="param">
  
    <li>
      
        <span class='name'>x</span>
      
      
        <span class='type'>(<tt>String</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>parameter x</div>
      
    </li>
  
    <li>
      
        <span class='name'>y</span>
      
      
        <span class='type'>(<tt>Boolean</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>parameter y</div>
      
    </li>
  
    <li>
      
        <span class='name'>z</span>
      
      
        <span class='type'>(<tt>Boolean</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>parameter z</div>
      
    </li>
  
</ul>


</div>
      </li>
    
  </ul>


</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


11</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 11</span>

def m(*args) end</pre>
    </td>
  </tr>
</table>
</div>
  </div>
</div>
