<meta charset="<%= charset %>">
<meta name="viewport" content="width=device-width, initial-scale=1.0">
<title>
  <%= h @page_title %>
  <% if options.title && @page_title != options.title %>
    &mdash; <%= h options.title %>
  <% end %>
</title>
<% stylesheets.each do |stylesheet| %>
  <link rel="stylesheet" href="<%= mtime_url(stylesheet) %>" type="text/css">
<% end %>
<%= erb :script_setup %>
<% javascripts.each do |javascript| %>
  <script type="text/javascript" charset="utf-8" src="<%= mtime_url(javascript) %>"></script>
<% end %>
