<p class="note private">
  <strong>This <%= object.type %> is part of a private API.</strong>
  You should avoid using this <%= object.type %> if possible, as it may be removed or be changed in the future.
</p>
