<h1>Module: A
  
  
  
</h1>
<div class="box_info">
  

  
  
  
  
  <dl>
      <dt>Includes:</dt>
      <dd>B, BaseMod</dd>
  </dl>
  
  

  
  <dl>
    <dt>Included in:</dt>
    <dd>TMP, TMP2</dd>
  </dl>
  

  
  <dl>
    <dt>Defined in:</dt>
    <dd>(stdin)</dd>
  </dl>
  
</div>

<h2>Overview</h2><div class="docstring">
  <div class="discussion">
    Comments

  </div>
</div>
<div class="tags">
  

</div><h2>Defined Under Namespace</h2>
<p class="children">
  
    
      <strong class="modules">Modules:</strong> Z
    
  
    
      <strong class="classes">Classes:</strong> Q, X, Y
    
  
</p>

  <h2>
    Constant Summary

    <small><a href="#" class="constants_summary_toggle">collapse</a></small>
  </h2>
  
    <dl class="constants">
      
        <dt id="CONSTANT-constant" class="">CONSTANT =
          <div class="docstring">
  <div class="discussion">
    A long docstring for the constant. With extra text
and newlines.

  </div>
</div>
<div class="tags">
  

</div>
        </dt>
        <dd><pre class="code">&#39;value&#39;</pre></dd>
      
        <dt id="cvar-classvariable" class="deprecated">@@cvar =
          <div class="docstring">
  <div class="discussion">
    <div class="note deprecated"><strong>Deprecated.</strong> <div class='inline'></div></div>


  </div>
</div>
<div class="tags">
  

</div>
        </dt>
        <dd><pre class="code">&#39;value&#39;</pre></dd>
      
    </dl>
  




  <h2>Instance Attribute Summary <small><a href="#" class="summary_toggle">collapse</a></small></h2>
  <ul class="summary">
    
      <li class="public ">
  <span class="summary_signature">
    
      <a  title="#attr1 (instance method)">#<strong>attr1</strong>  </a>
    

    
  </span>
  
  
  
    
    
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>Returns the value of attribute attr1.</div></span>
  
</li>

    
      <li class="public ">
  <span class="summary_signature">
    
      <a  title="#attr2 (instance method)">#<strong>attr2</strong>  </a>
    

    
  </span>
  
  
  
    
      <span class="note title readonly">readonly</span>
    
    
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>Returns the value of attribute attr2.</div></span>
  
</li>

    
      <li class="public ">
  <span class="summary_signature">
    
      <a  title="#attr3 (instance method)">#<strong>attr3</strong>  </a>
    

    
  </span>
  
  
  
    
    
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

    
      <li class="public ">
  <span class="summary_signature">
    
      <a  title="#attr4= (instance method)">#<strong>attr4</strong>  </a>
    

    
  </span>
  
  
  
    
    
      <span class="note title writeonly">writeonly</span>
    
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>Sets the attribute attr4.</div></span>
  
</li>

    
  </ul>



  
  
  <h3 class="inherited">Attributes included from BaseMod</h3>
  <p class="inherited">#base_attr1, #base_attr2, #base_attr3</p>


  
    <h2>
      Class Method Summary
      <small><a href="#" class="summary_toggle">collapse</a></small>
    </h2>

    <ul class="summary">
      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="a (class method)">.<strong>a</strong>  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

      
    </ul>
  
    <h2>
      Instance Method Summary
      <small><a href="#" class="summary_toggle">collapse</a></small>
    </h2>

    <ul class="summary">
      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#a (instance method)">#<strong>a</strong>  </a>
    

    
      (also: #b)
    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#test_multi_overload (instance method)">#<strong>test_multi_overload</strong>(*args)  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#test_overload (instance method)">#<strong>test_overload</strong>(a)  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>hello2.</div></span>
  
</li>

      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#void_meth (instance method)">#<strong>void_meth</strong>  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

      
    </ul>
  


  
  
  
  
  
  
  
  
  <h3 class="inherited">Methods included from B</h3>
  <p class="inherited">#c, #d</p>

  
  
  
  
  
  
  <div id="instance_attr_details" class="attr_details">
    <h2>Instance Attribute Details</h2>
    
      
      <span id="attr1=-instance_method"></span>
      <div class="method_details first">
  <h3 class="signature first" id="attr1-instance_method">
  
    #<strong>attr1</strong>  
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    Returns the value of attribute attr1.

  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


17
18
19</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 17</span>

def attr1
  @attr1
end</pre>
    </td>
  </tr>
</table>
</div>
    
      
      <span id=""></span>
      <div class="method_details ">
  <h3 class="signature " id="attr2-instance_method">
  
    #<strong>attr2</strong>   <span class="extras">(readonly)</span>
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    Returns the value of attribute attr2.

  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


18
19
20</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 18</span>

def attr2
  @attr2
end</pre>
    </td>
  </tr>
</table>
</div>
    
      
      <span id="attr3=-instance_method"></span>
      <div class="method_details ">
  <h3 class="signature " id="attr3-instance_method">
  
    
      <span class="overload">#<strong>attr3</strong>  &#x21d2; <tt>String</tt> </span>
    
      <span class="overload">#<strong>attr3=</strong>(value)  </span>
    
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    

  </div>
</div>
<div class="tags">
  
  <p class="tag_title">Overloads:</p>
  <ul class="overload">
    
      
      <li class="overload_item">
        <span class="signature">#<strong>attr3</strong>  &#x21d2; <tt>String</tt> </span>
        <div class="docstring">
  <div class="discussion">
    Returns a string.

  </div>
</div>
<div class="tags">
  
<p class="tag_title">Returns:</p>
<ul class="return">
  
    <li>
      
      
        <span class='type'>(<tt>String</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>a string</div>
      
    </li>
  
</ul>

</div>
      </li>
    
      
      <li class="overload_item">
        <span class="signature">#<strong>attr3=</strong>(value)  </span>
        <div class="docstring">
  <div class="discussion">
    <p class="note returns_void">This method returns an undefined value.</p>

  </div>
</div>
<div class="tags">
  <p class="tag_title">Parameters:</p>
<ul class="param">
  
    <li>
      
        <span class='name'>value</span>
      
      
        <span class='type'>(<tt>String</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>sets the string</div>
      
    </li>
  
</ul>


</div>
      </li>
    
  </ul>


</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


25
26
27</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 25</span>

def attr3
  @attr3
end</pre>
    </td>
  </tr>
</table>
</div>
    
      
      <span id=""></span>
      <div class="method_details ">
  <h3 class="signature " id="attr4=-instance_method">
  
    #<strong>attr4=</strong>(value)   <span class="extras">(writeonly)</span>
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    Sets the attribute attr4

  </div>
</div>
<div class="tags">
  <p class="tag_title">Parameters:</p>
<ul class="param">
  
    <li>
      
        <span class='name'>value</span>
      
      
        <span class='type'></span>
      
      
      
        &mdash;
        <div class='inline'>the value to set the attribute attr4 to.</div>
      
    </li>
  
</ul>


</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


27
28
29</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 27</span>

def attr4=(value)
  @attr4 = value
end</pre>
    </td>
  </tr>
</table>
</div>
    
  </div>


  <div id="class_method_details" class="method_details_list">
    <h2>Class Method Details</h2>

    
      <div class="method_details first">
  <h3 class="signature first" id="a-class_method">
  
    .<strong>a</strong>  
  

  

  
</h3><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


29</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 29</span>

def self.a; end</pre>
    </td>
  </tr>
</table>
</div>
    
  </div>

  <div id="instance_method_details" class="method_details_list">
    <h2>Instance Method Details</h2>

    
      <div class="method_details first">
  <h3 class="signature first" id="a-instance_method">
  
    #<strong>a</strong>  
  

  
    <span class="aliases">Also known as:
    <span class="names"><span id='b-instance_method'>b</span></span>
    </span>
  

  
</h3><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


30</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 30</span>

def a; end</pre>
    </td>
  </tr>
</table>
</div>
    
      <div class="method_details ">
  <h3 class="signature " id="test_multi_overload-instance_method">
  
    
      <span class="overload">#<strong>test_multi_overload</strong>(a)  </span>
    
      <span class="overload">#<strong>test_multi_overload</strong>(a, b)  </span>
    
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    

  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


40</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 40</span>

def test_multi_overload(*args) end</pre>
    </td>
  </tr>
</table>
</div>
    
      <div class="method_details ">
  <h3 class="signature " id="test_overload-instance_method">
  
    #<strong>test_overload</strong>(a)  
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    

  </div>
</div>
<div class="tags">
  
  <div class="docstring">
  <div class="discussion">
    hello2

  </div>
</div>
<div class="tags">
  <p class="tag_title">Parameters:</p>
<ul class="param">
  
    <li>
      
        <span class='name'>a</span>
      
      
        <span class='type'>(<tt>String</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>hi</div>
      
    </li>
  
</ul>


</div>


</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


36</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 36</span>

def test_overload(*args) end</pre>
    </td>
  </tr>
</table>
</div>
    
      <div class="method_details ">
  <h3 class="signature " id="void_meth-instance_method">
  
    #<strong>void_meth</strong>  
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    <p class="note returns_void">This method returns an undefined value.</p>

  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


43</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 43</span>

def void_meth; end</pre>
    </td>
  </tr>
</table>
</div>
    
  </div>
