<% scopes(method_listing(false)) do |list, scope| %>
  <div id="<%= scope %>_method_details" class="method_details_list">
    <h2><%= scope.to_s.capitalize %> Method Details</h2>

    <% list.each_with_index do |meth, i| %>
      <%= yieldall :object => meth, :owner => object, :index => i %>
    <% end %>
  </div>
<% end %>
