<% if method_listing.size > 0 %>
  <% groups(method_listing) do |list, name| %>
    <h2>
      <%= name %>
      <small><a href="#" class="summary_toggle">collapse</a></small>
    </h2>

    <ul class="summary">
      <% list.each do |meth| %>
        <%= yieldall :item => meth %>
      <% end %>
    </ul>
    <% (inherited_methods_by_group[name] || {}).each do |superclass, meths| %>
      <h3 class="inherited">Methods <%= superclass.type == :class ? 'inherited' : 'included' %> from <%= linkify superclass %></h3>
      <p class="inherited"><%= meths.sort_by {|o| o.name.to_s }.map {|m|
        mname = m.name(true)
        mname = mname.gsub(/^#/, '') if superclass.type == :module && object.class_mixins.include?(superclass)
        linkify(m, mname)
      }.join(", ") %></p>
    <% end %>
  <% end %>
<% end %>