<h1>Module: A



</h1>
<div class="box_info">











  <dl>
    <dt>Defined in:</dt>
    <dd>(stdin)</dd>
  </dl>

</div>



    <h2>
      Foo
      <small><a href="#" class="constants_summary_toggle">collapse</a></small>
    </h2>

    <dl class="constants">

        <dt id="FOO-constant" class="">FOO =

        </dt>
        <dd><pre class="code">1</pre></dd>

        <dt id="BAR-constant" class="deprecated">BAR =
          <div class="docstring">
  <div class="discussion">
    <div class="note deprecated"><strong>Deprecated.</strong> <div class='inline'></div></div>


  </div>
</div>
<div class="tags">


</div>
        </dt>
        <dd><pre class="code">2</pre></dd>

    </dl>

    <h2>
      Bar
      <small><a href="#" class="constants_summary_toggle">collapse</a></small>
    </h2>

    <dl class="constants">

        <dt id="BAZ-constant" class="">BAZ =

        </dt>
        <dd><pre class="code">3</pre></dd>

    </dl>

    <h2>
      Constant Summary
      <small><a href="#" class="constants_summary_toggle">collapse</a></small>
    </h2>

    <dl class="constants">

        <dt id="WORLD-constant" class="">WORLD =

        </dt>
        <dd><pre class="code">4</pre></dd>

    </dl>