<h1><%= format_object_title(object) %>
  <% if object.has_tag?(:abstract) %><span class="abstract note title">Abstract</span><% end %>
  <% if object.has_tag?(:deprecated) %><span class="deprecated note title">Deprecated</span><% end %>
  <% if object.has_tag?(:api) && object.tag(:api).text == 'private' %><span class="private note title">Private</span><% end %>
</h1>
