<h1>Class: A
  
  
  
</h1>
<div class="box_info">
  
  <dl>
    <dt>Inherits:</dt>
    <dd>
      <span class="inheritName">Object</span>
      
        <ul class="fullTree">
          <li>Object</li>
          
            <li class="next">A</li>
          
        </ul>
        <a href="#" class="inheritanceTree">show all</a>
      
    </dd>
  </dl>
  

  
  
  <dl>
      <dt>Extended by:</dt>
      <dd>Bar</dd>
  </dl>
  
  
  
  <dl>
      <dt>Includes:</dt>
      <dd>BarFooBar, Baz::ABC, Baz::XYZ, Foo</dd>
  </dl>
  
  

  

  
  <dl>
    <dt>Defined in:</dt>
    <dd>(stdin)</dd>
  </dl>
  
</div>





  <h2>Instance Attribute Summary <small><a href="#" class="summary_toggle">collapse</a></small></h2>
  <ul class="summary">
    
      <li class="public ">
  <span class="summary_signature">
    
      <a  title="#bar_attr (instance method)">#<strong>bar_attr</strong>  </a>
    

    
  </span>
  
    <span class="note title not_defined_here">
      included
      from Foo
    </span>
  
  
  
    
    
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>Docs for bar_attr.</div></span>
  
</li>

    
  </ul>




  
    <h2>
      Booya
      <small><a href="#" class="summary_toggle">collapse</a></small>
    </h2>

    <ul class="summary">
      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#baz (class method)">.<strong>baz</strong>  </a>
    

    
  </span>
  
    <span class="note title not_defined_here">
      extended
      from Bar
    </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>Docs for baz in Booya group.</div></span>
  
</li>

      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#baz_abc (instance method)">#<strong>baz_abc</strong>  </a>
    

    
  </span>
  
    <span class="note title not_defined_here">
      included
      from Baz::ABC
    </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

      
    </ul>
  
    <h2>
      Instance Method Summary
      <small><a href="#" class="summary_toggle">collapse</a></small>
    </h2>

    <ul class="summary">
      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#foo (instance method)">#<strong>foo</strong>  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>This method is in A.</div></span>
  
</li>

      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#xyz (instance method)">#<strong>xyz</strong>  </a>
    

    
  </span>
  
    <span class="note title not_defined_here">
      included
      from Foo
    </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>Docs for xyz.</div></span>
  
</li>

      
    </ul>
  


  
  
  
  
  
  
  
  
  
  
  
  
  <h3 class="inherited">Methods included from Baz::XYZ</h3>
  <p class="inherited">#baz_xyz</p>

  
  
  
  
  
  
  
  
  <h3 class="inherited">Methods included from BarFooBar</h3>
  <p class="inherited">#bar_foo_bar</p>

  
  
  <div id="instance_attr_details" class="attr_details">
    <h2>Instance Attribute Details</h2>
    
      
      <span id="bar_attr=-instance_method"></span>
      <div class="method_details first">
  <h3 class="signature first" id="bar_attr-instance_method">
  
    #<strong>bar_attr</strong>  
  

  

  
    <span class="not_defined_here">
      Originally defined in module
        Foo
    </span>
  
</h3><div class="docstring">
  <div class="discussion">
    Docs for bar_attr

  </div>
</div>
<div class="tags">
  

</div>
</div>
    
  </div>


  <div id="class_method_details" class="method_details_list">
    <h2>Class Method Details</h2>

    
      <div class="method_details first">
  <h3 class="signature first" id="baz-class_method">
  
    .<strong>baz</strong>  
  

  

  
    <span class="not_defined_here">
      Originally defined in module
        Bar
    </span>
  
</h3><div class="docstring">
  <div class="discussion">
    Docs for baz in Booya group

  </div>
</div>
<div class="tags">
  

</div>
</div>
    
  </div>

  <div id="instance_method_details" class="method_details_list">
    <h2>Instance Method Details</h2>

    
      <div class="method_details first">
  <h3 class="signature first" id="baz_abc-instance_method">
  
    #<strong>baz_abc</strong>  
  

  

  
    <span class="not_defined_here">
      Originally defined in module
        Baz::ABC
    </span>
  
</h3>
</div>
    
      <div class="method_details ">
  <h3 class="signature " id="foo-instance_method">
  
    #<strong>foo</strong>  
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    This method is in A

  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


3</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 3</span>

def foo; end</pre>
    </td>
  </tr>
</table>
</div>
    
      <div class="method_details ">
  <h3 class="signature " id="xyz-instance_method">
  
    #<strong>xyz</strong>  
  

  

  
    <span class="not_defined_here">
      Originally defined in module
        Foo
    </span>
  
</h3><div class="docstring">
  <div class="discussion">
    Docs for xyz

  </div>
</div>
<div class="tags">
  

</div>
</div>
    
  </div>

