<style type="text/css">
  <%= @css_data %>
  html { width: auto; height: auto; }
  body { display: block; margin: 1rem; width: auto; }
</style>
<script type="text/javascript">
  <%= @js_data %>
</script>
