<meta charset="UTF-8">
<title>YARD Documentation Server <%= YARD::VERSION %> - Library Listing</title>
<link rel="stylesheet" href="<%= abs_url('css', 'style.css') %>?<%= mtime('css/style.css') %>" type="text/css" media="screen" charset="utf-8">
<link rel="stylesheet" href="<%= abs_url('css', 'custom.css') %>?<%= mtime('css/custom.css') %>" type="text/css" media="screen" charset="utf-8">
<style type="text/css" media="screen">
  ul { list-style: circle inside none; padding: 0; }
  li { font-size: 1.2em; line-height: 1.4em; padding: 3px 5px; }
</style>
