<h2>Tag Listing</h2>
<p>The following are a brief list of tags that can be used in YARD formatted
  documentation. See <%= link_file "docs/Tags.md" %> for information on using tags.</p>
<ul id="tags" class="index_inline_list">
<% n = 1 %>
<% collect_tags.each do |tag| %>
  <li class="r<%= n %>"><%= tag_link(tag) %></li>
  <% n = n == 2 ? 1 : 2 %>
<% end %>
</ul>
<div class="clear"></div>
