<h1>Module: A
  
  
  
</h1>
<div class="box_info">
  

  
  
  
  
  

  

  
  <dl>
    <dt>Defined in:</dt>
    <dd>(stdin)</dd>
  </dl>
  
</div>





  <h2>Foo <small><a href="#" class="summary_toggle">collapse</a></small></h2>
  <ul class="summary">
    
      <li class="public ">
  <span class="summary_signature">
    
      <a  title="#foo_attr (instance method)">#<strong>foo_attr</strong>  </a>
    

    
  </span>
  
  
  
    
    
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>Returns the value of attribute foo_attr.</div></span>
  
</li>

    
  </ul>




  
    <h2>
      Foo
      <small><a href="#" class="summary_toggle">collapse</a></small>
    </h2>

    <ul class="summary">
      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="bar (class method)">.<strong>bar</strong>  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#foo (instance method)">#<strong>foo</strong>  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

      
    </ul>
  
    <h2>
      Bar
      <small><a href="#" class="summary_toggle">collapse</a></small>
    </h2>

    <ul class="summary">
      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#baz (instance method)">#<strong>baz</strong>  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

      
    </ul>
  
    <h2>
      Class Method Summary
      <small><a href="#" class="summary_toggle">collapse</a></small>
    </h2>

    <ul class="summary">
      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="baz (class method)">.<strong>baz</strong>  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

      
    </ul>
  


  <div id="instance_attr_details" class="attr_details">
    <h2>Instance Attribute Details</h2>
    
      
      <span id="foo_attr=-instance_method"></span>
      <div class="method_details first">
  <h3 class="signature first" id="foo_attr-instance_method">
  
    #<strong>foo_attr</strong>  
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    Returns the value of attribute foo_attr.

  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


3
4
5</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 3</span>

def foo_attr
  @foo_attr
end</pre>
    </td>
  </tr>
</table>
</div>
    
  </div>


  <div id="class_method_details" class="method_details_list">
    <h2>Class Method Details</h2>

    
      <div class="method_details first">
  <h3 class="signature first" id="bar-class_method">
  
    .<strong>bar</strong>  
  

  

  
</h3><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


5</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 5</span>

def self.bar; end</pre>
    </td>
  </tr>
</table>
</div>
    
      <div class="method_details ">
  <h3 class="signature " id="baz-class_method">
  
    .<strong>baz</strong>  
  

  

  
</h3><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


12</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 12</span>

def self.baz; end</pre>
    </td>
  </tr>
</table>
</div>
    
  </div>

  <div id="instance_method_details" class="method_details_list">
    <h2>Instance Method Details</h2>

    
      <div class="method_details first">
  <h3 class="signature first" id="baz-instance_method">
  
    #<strong>baz</strong>  
  

  

  
</h3><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


8</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 8</span>

def baz; end</pre>
    </td>
  </tr>
</table>
</div>
    
      <div class="method_details ">
  <h3 class="signature " id="foo-instance_method">
  
    #<strong>foo</strong>  
  

  

  
</h3><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


4</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 4</span>

def foo; end</pre>
    </td>
  </tr>
</table>
</div>
    
  </div>
