<% if object.tags(:overload).size == 1 %>
<%= wrap(yieldall(:object => object.tag(:overload))) %>
<% elsif object.has_tag?(:overload) %>
Overloads:
----------

<% object.tags(:overload).each do |overload| %>
<% if overload.tags.size > 0 %>
<%= hr %>

<%= indent signature(overload) %>

<%= hr %>

<%= indent(wrap(yieldall(:object => overload))) %>

<% end %>
<% end %>
<% end %>
