<% even_odd = 'odd'  %>
<% @items.each do |item| %>
  <li id="object_<%= item.path %>" class="<%= even_odd %>">
    <div class="item"><span class="object_link"><%= link_file item %></span></div>
  </li>
  <% even_odd = (even_odd == 'even' ? 'odd' : 'even') %>
<% end %>
