<h1>Module: A
  
  
  
</h1>
<div class="box_info">
  

  
  
  
  
  <dl>
      <dt>Includes:</dt>
      <dd>B</dd>
  </dl>
  
  

  

  
  <dl>
    <dt>Defined in:</dt>
    <dd>(stdin)</dd>
  </dl>
  
</div>


  <h2>
    Constant Summary

    <small><a href="#" class="constants_summary_toggle">collapse</a></small>
  </h2>
  
    <dl class="constants">
      
        <dt id="FOO-constant" class="">FOO =
          
        </dt>
        <dd><pre class="code">2</pre></dd>
      
    </dl>
  




  <h2>Instance Attribute Summary <small><a href="#" class="summary_toggle">collapse</a></small></h2>
  <ul class="summary">
    
      <li class="public ">
  <span class="summary_signature">
    
      <a  title="#bar (instance method)">#<strong>bar</strong>  </a>
    

    
  </span>
  
  
  
    
      <span class="note title readonly">readonly</span>
    
    
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'>Returns the value of attribute bar.</div></span>
  
</li>

    
  </ul>




  
    <h2>
      Instance Method Summary
      <small><a href="#" class="summary_toggle">collapse</a></small>
    </h2>

    <ul class="summary">
      
        <li class="public ">
  <span class="summary_signature">
    
      <a  title="#foo (instance method)">#<strong>foo</strong>  </a>
    

    
  </span>
  
  
  
  
  
  
  

  
    <span class="summary_desc"><div class='inline'></div></span>
  
</li>

      
    </ul>
  


  
  
  
  
  
  
  <div id="instance_attr_details" class="attr_details">
    <h2>Instance Attribute Details</h2>
    
      
      <span id=""></span>
      <div class="method_details first">
  <h3 class="signature first" id="bar-instance_method">
  
    #<strong>bar</strong>   <span class="extras">(readonly)</span>
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    Returns the value of attribute bar.

  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


11
12
13</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 11</span>

def bar
  @bar
end</pre>
    </td>
  </tr>
</table>
</div>
    
  </div>


  <div id="instance_method_details" class="method_details_list">
    <h2>Instance Method Details</h2>

    
      <div class="method_details first">
  <h3 class="signature first" id="foo-instance_method">
  
    #<strong>foo</strong>  
  

  

  
</h3><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


10</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 10</span>

def foo; end</pre>
    </td>
  </tr>
</table>
</div>
    
  </div>
