<script>
document.getElementById("<%= anchor_for(object) %>").insertAdjacentHTML(
  "afterbegin",
  '<a class="permalink" href="<%= abs_url base_path(router.docs_prefix) %>/<%= urlencode serializer.serialized_path(object) %>">permalink</a>'
);
</script>
