<% groups(attr_listing, "Attribute") do |list, name| %>
  <h2><%= name %> <small><a href="#" class="summary_toggle">collapse</a></small></h2>
  <ul class="summary">
    <% list.each do |meth| %>
      <%= yieldall :item => meth %>
    <% end %>
  </ul>
<% end %>
