<%= title_align_right format_object_title(object) %>

<%= align_right "(Defined in: #{object.file})" %>
<% if object.aliases.size > 0 %>

<%= align_right "(Also known as: #{object.aliases.map {|o| o.name(true).to_s }.join(',')})" %>
<% end %>

<%= yieldall %>

