<h1>Method: #m</h1>
<div class="box_info">
  <dl>
    <dt class="">Defined in:</dt>
    <dd class="">
    (stdin)
    </dd>
  </dl>
</div>

<div class="method_details_list">
  <div id="method_details">
    <div class="method_details first">
  <h3 class="signature first" id="m-instance_method">
  
    #<strong>m</strong>(*args)  &#x21d2; <tt>void</tt> 
  

  

  
</h3><div class="docstring">
  <div class="discussion">
    <p class="note returns_void">This method returns an undefined value.</p>

  </div>
</div>
<div class="tags">
  

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


2</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 2</span>

def m(*args) end</pre>
    </td>
  </tr>
</table>
</div>
  </div>
</div>
