<div id="footer">
  Generated on <%= Time.now.strftime("%c") %> by
  <a href="https://yardoc.org" title="Yay! A Ruby Documentation Tool" target="_parent">yard</a>
  <%= YARD::VERSION %> (ruby-<%= RUBY_VERSION %>).
</div>
