<h1>Method: #m</h1>
<div class="box_info">
  <dl>
    <dt class="">Defined in:</dt>
    <dd class="">
    (stdin)
    </dd>
  </dl>
</div>

<div class="method_details_list">
  <div id="method_details">
    <div class="method_details first">
  <h3 class="signature first" id="m-instance_method">
  
    #<strong>m</strong>(x)  &#x21d2; <tt>String</tt>  <span class="extras">(private)</span>
  

  
    <span class="aliases">Also known as:
    <span class="names"><span id='x-instance_method'>x</span></span>
    </span>
  

  
</h3><div class="docstring">
  <div class="discussion">
    <div class="note deprecated"><strong>Deprecated.</strong> <div class='inline'>for great justice</div></div>
Comments

  </div>
</div>
<div class="tags">
  <p class="tag_title">Parameters:</p>
<ul class="param">
  
    <li>
      
        <span class='name'>x</span>
      
      
        <span class='type'>(<tt>Hash</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>the x argument</div>
      
    </li>
  
</ul>

  
    
    
    <p class="tag_title">Options Hash (<tt>x</tt>):</p>
    <ul class="option">
      
        <li>
          <span class="name">:key1</span>
          <span class="type">(<tt>String</tt>)</span>
          <span class="default">
            
              &mdash; default:
              <tt>default</tt>
            
          </span>
          
            &mdash; <div class='inline'>first key</div>
          
        </li>
      
        <li>
          <span class="name">:key2</span>
          <span class="type">(<tt>Symbol</tt>)</span>
          <span class="default">
            
          </span>
          
            &mdash; <div class='inline'>second key</div>
          
        </li>
      
    </ul>
  

<p class="tag_title">Returns:</p>
<ul class="return">
  
    <li>
      
      
        <span class='type'>(<tt>String</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>the result</div>
      
    </li>
  
</ul>
<p class="tag_title">Raises:</p>
<ul class="raise">
  
    <li>
      
      
        <span class='type'>(<tt>Exception</tt>)</span>
      
      
      
        &mdash;
        <div class='inline'>hi!</div>
      
    </li>
  
</ul>

</div><table class="source_code">
  <tr>
    <td>
      <pre class="lines">


9</pre>
    </td>
    <td>
      <pre class="code"><span class="info file"># File '(stdin)', line 9</span>

def m(x) end</pre>
    </td>
  </tr>
</table>
</div>
  </div>
</div>
